import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateFooterHelpSectionDto} from "./dto/create-footer-help-section.dto";
import {UpdateFooterHelpSectionDto} from "./dto/update-footer-help-section.dto";
import {FooterHelpSectionEntity} from "./entity/footer-help-section.entity";

@EntityRepository(FooterHelpSectionEntity)
export class FooterHelpSectionRepository extends Repository<FooterHelpSectionEntity> {

    async createEntity(dto: CreateFooterHelpSectionDto): Promise<FooterHelpSectionEntity | Error> {
        try {
            const {
                sectionText,
                sectionLink
            } = dto;
            const entity = this.create({
                sectionText,
                sectionLink
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<FooterHelpSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<FooterHelpSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateFooterHelpSectionDto): Promise<FooterHelpSectionEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(FooterHelpSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}