import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateFooterServiceSectionDto} from "./dto/create-footer-service-section.dto";
import {UpdateFooterServiceSectionDto} from "./dto/update-footer-service-section.dto";
import {FooterServiceSectionEntity} from "./entity/footer-service-section.entity";

@EntityRepository(FooterServiceSectionEntity)
export class FooterServiceSectionRepository extends Repository<FooterServiceSectionEntity> {

    async createEntity(dto: CreateFooterServiceSectionDto): Promise<FooterServiceSectionEntity | Error> {
        try {
            const {
                sectionText,
                sectionLink
            } = dto;
            const entity = this.create({
                sectionText,
                sectionLink
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<FooterServiceSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<FooterServiceSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateFooterServiceSectionDto): Promise<FooterServiceSectionEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(FooterServiceSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}