import { Body, Controller, Delete, Get, Param, Patch, Post, UploadedFiles, UseGuards, UseInterceptors } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { InstagramServiceService } from "./instagram-service.service";
import { CreateInstagramServiceDto } from "./dto/create-instagram-service.dto";
import { UpdateInstagramServiceDto } from "./dto/update-instagram-service.dto";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";

@Controller({ path: "instagram-service", version: Constants.API_VERSION_1 })
export class InstagramServiceController {
  constructor(private readonly service: InstagramServiceService) {
  }

  @Post()
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "media_url", maxCount: 1 }
    ]
  ))
  create(@Body() dto: CreateInstagramServiceDto,
    @UploadedFiles() files: {
      media_url: UploadedMulterFileI
    }) {
    return this.service.create(dto, files);
  }

  @Get()
  // @UseGuards(AdminAuthGuard)
  findAll() {
    return this.service.findAll();
  }

  @Get(":id")
  // @UseGuards(AdminAuthGuard)
  findOne(@Param("id") id: string) {
    return this.service.findOne(id);
  }

  @Patch(":id")
  @UseGuards(AdminAuthGuard)
  @UseInterceptors(FileFieldsInterceptor(
    [
      { name: "media_url", maxCount: 1 }
    ]
  ))
  update(@Param("id") id: string, @Body() dto: UpdateInstagramServiceDto,
    @UploadedFiles() files: {
      media_url: UploadedMulterFileI
    }) {
    return this.service.update(id, dto, files);
  }

  @Delete(":id")
  // @UseGuards(AdminAuthGuard)
  async remove(@Param("id") id: string) {
    return await this.service.remove(id);
  }
}
