import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";
import { LatestCollectionSectionRepository } from "./latest-collection-section.repository";
import { CreateLatestCollectionSectionDto } from "./dto/create-latest-collection-section.dto";
import { LatestCollectionSectionEntity } from "./entities/latest-collection-section.entity";
import { UpdateLatestCollectionSectionDto } from "./dto/update-latest-collection-section.dto";

@Injectable()
export class LatestCollectionSectionService {
    constructor(
        @InjectRepository(LatestCollectionSectionRepository)
        private readonly repository: LatestCollectionSectionRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateLatestCollectionSectionDto,
        files: {
            image: UploadedMulterFileI
        }
    ): Promise<LatestCollectionSectionEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "LatestCollectionSection/");
            dto.image = image;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<LatestCollectionSectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<LatestCollectionSectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async update(
        id: string,
        dto: UpdateLatestCollectionSectionDto,
        files: {
            image: UploadedMulterFileI
        }): Promise<LatestCollectionSectionEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "HeroSectionSlider/");
            dto.image = image;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }
}
