import {EntityRepository, getConnection, Repository} from "typeorm";
import {SocialMediaLinkEntity} from "./entities/social-media-link.entity";
import {CreateSocialMediaLinkDto} from "./dto/create-social-media-link.dto";
import {UpdateSocialMediaLinkDto} from "./dto/update-social-media-link.dto";

@EntityRepository(SocialMediaLinkEntity)
export class SocialMediaLinkRepository extends Repository<SocialMediaLinkEntity> {
    async createEntity(dto: CreateSocialMediaLinkDto): Promise<SocialMediaLinkEntity | Error> {
        try {
            const {
                logo,
                link,
            } = dto;
            const entity = this.create({
                logo,
                link
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<SocialMediaLinkEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<SocialMediaLinkEntity[] | Error> {
        try {
            return await this.find({
                where: {
                    ...query,
                    isDeleted: false
                },
                order: {
                    createdAt: "DESC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<SocialMediaLinkEntity[] | Error> {
        try {
            return await this.find({
                order: {
                    createdAt: "ASC"
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(SocialMediaLinkEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateSocialMediaLinkDto): Promise<SocialMediaLinkEntity | Error> {
        try {
            await this.update(id, {
                logo: dto.logo,
                link: dto.link
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }
}