import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {PaymentMethodService} from './payment-method.service';
import {CreatePaymentMethodDto} from './dto/create-payment-method.dto';
import {UpdatePaymentMethodDto} from './dto/update-payment-method.dto';

@Controller({path: 'payment-method', version: Constants.API_VERSION_1})
export class PaymentMethodController {
    constructor(private readonly service: PaymentMethodService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreatePaymentMethodDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdatePaymentMethodDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
