import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductColorVariantsDto } from "./dto/create-product-color-variants.dto";
import { UpdateProductColorVariantsDto } from "./dto/update-product-color-variants.dto";
import { ProductColorVariants } from "./entities/product-color-variants.entity";

@EntityRepository(ProductColorVariants)
export class ProductColorVariantsRepository extends Repository<ProductColorVariants> {

    async createEntity(dto: CreateProductColorVariantsDto): Promise<ProductColorVariants | Error> {
        try {
            const { productId, productName, colorId, colorName, colorCode } = dto;
            const entity = this.create({
                productId,
                productName,
                colorId,
                colorName,
                colorCode
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductColorVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductColorVariants | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductColorVariantsDto): Promise<ProductColorVariants | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductColorVariants)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    // async findByName(name: string): Promise<ProductColorVariants[] | Error> {
    //     try {
    //         return await this.find({
    //             where: { 
    //                 name: name, 
    //                 isDeleted: false 
    //             }
    //         });
    //     }
    //     catch (e) {
    //         return e;
    //     }
    // }

    async deleteColorVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductColorVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertColorVariant(newColor: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductColorVariants)
                .values(newColor)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductColorValues(id: any, newColorEntity: any): Promise<any | Error> {
        try {
            const deleteCurrent = await this.deleteColorVariant(id);
            await this.bulkInsertColorVariant(newColorEntity)
        } catch (e) {
            return e;
        }
    }
}