import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { CreateProductOfferDto } from "./dto/create-product-offer.dto";
import { UpdateProductOfferDto } from "./dto/update-product-offer.dto";
import { ProductOffer } from "./entities/product-offer.entity";

@EntityRepository(ProductOffer)
export class ProductOfferRepository extends Repository<ProductOffer> {

    async createEntity(dto: CreateProductOfferDto): Promise<any | Error> {
        try {
            const entity = this.create({
                image: dto.image,
                title: dto.title,
                description: dto.description
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<ProductOffer[] | Error> {
        try {
            return await this.find({
                order: {
                    createdAt: -1
                },
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductOffer[] | Error> {
        try {
            return await this.find({
                where: {
                    id: id
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntityBySlug(urlSlug: string): Promise<ProductOffer[] | Error> {
        try {
            return await this.find({
                where: {
                    urlSlug: urlSlug
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductOffer)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async updateProductOffer(id: string, dto: UpdateProductOfferDto): Promise<ProductOffer | Error> {
        try {
            await this.update(id, {
                image: dto.image,
                title: dto.title,
                description: dto.description
            });
            return await this.findOne({ where: { id: id } });
        } catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductOffer[] | Error> {
        try {
            return await this.find({
                where: {
                    title: name,
                    isDeleted: false
                }
            });
        }
        catch (e) {
            return e;
        }
    }

}