import { EntityRepository, getConnection, Repository } from "typeorm";
import { ProductSaleVariants } from "./entities/product-sale-variants.entity";
import { CreateProductSaleVariantsDto } from "./dto/create-product-sale-variants.dto";
import { UpdateProductSaleVariantsDto } from "./dto/update-product-sale-variants.dto";

@EntityRepository(ProductSaleVariants)
export class ProductSaleVariantsRepository extends Repository<ProductSaleVariants> {

    async createEntity(dto: CreateProductSaleVariantsDto): Promise<ProductSaleVariants | Error> {
        try {
            const { productId, productName, saleId, saleName } = dto;
            const entity = this.create({
                productId,
                productName,
                saleId,
                saleName
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductSaleVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ProductSaleVariants | Error> {
        try {
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateProductSaleVariantsDto): Promise<ProductSaleVariants | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        }
        catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ProductSaleVariants)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async findByName(name: string): Promise<ProductSaleVariants[] | Error> {
        try {
            return await this.find({
                where: { 
                    saleName: name, 
                    isDeleted: false 
                }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteSaleVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductSaleVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertSaleVariant(newSale: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductSaleVariants)
                .values(newSale)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductSaleValues(id: any, newSaleEntity: any): Promise<any | Error> {
        try {
            const deleteCurrent = await this.deleteSaleVariant(id);
            await this.bulkInsertSaleVariant(newSaleEntity)
        } catch (e) {
            return e;
        }
    }
}