import { Between, EntityRepository, getConnection, getManager, getRepository, MoreThan, Repository } from "typeorm";
import { Product } from "./entity/product.entity";
import { ProductCategory } from "../product-category/entities/product-category.entity";
import { ProductSizeVariants } from "../product-size-variants/entities/product-size-variants.entity";


@EntityRepository(Product)
export class ProductsRepository extends Repository<Product> {

    async createEntity(
        dto: any
    ): Promise<any | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);

            return entity;
        } catch (e) {
            return e;
        }
    }


    async updateEntity(
        id: string,
        dto: any
    ): Promise<Product | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }


    async findAllEntities(take: number, page: number): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    take, skip: take * (page - 1),
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false },
                }
            );
        } catch (e) {
            return e;
        }
    }

    async findEntitiesByLimit(count: number): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    take: count,
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false },
                    relations: ['photos', 'productsubCategory', 'productParentCategory'],
                },
            );
        } catch (e) {
            return e;
        }
    }

    async findProductByBrand(brandName: string, take: number, page: number, categoryId: String,
        maxPrice: number,
        minPrice: number,
        thickness: any, sortBy: any): Promise<any | Error> {
        try {

            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.firstCategoryId = '${categoryId}'`;
            }
            if (brandName) {
                //(brandName);
                if (whereQuery.length > 3) {
                    whereQuery += `and p.brandName = '${brandName}'`;
                } else {
                    whereQuery += ` p.brandName = '${brandName}'`;
                }

            }
            if (maxPrice && minPrice) {
                whereQuery += `and (p.price >= ${minPrice} and p.price <= ${maxPrice})`;
            }
            // if (color) {
            //     var result = '(';
            //     if (color.length > 0) {
            //         color = color.split(",")
            //     }
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += ` and c.color IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` c.color IN ${result} `;
            //     }

            // }
            // if (size) {
            //     var result = '(';
            //     if (size.length > 0) {
            //         size = size.split(",")
            //     }
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += `and s.size IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` s.size IN ${result} `;
            //     }

            // }
            if (thickness) {
                var result = '(';
                if (thickness.length > 0) {
                    thickness = thickness.split(",")
                }
                thickness.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.thickness IN ${result} `;
                } else {
                    whereQuery += ` t.thickness IN ${result} `;
                }

            }

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 `;
            } else {
                whereQuery += ` p.isDeleted=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }


            let sql = `SELECT p.* FROM products as p 
                        LEFT JOIN thickness_variant as t on t.productId=p.id
                        ${conditionQuery} GROUP BY p.id limit ${offset}, ${take};`
            let sqlTotal = `SELECT p.* FROM products as p 
                            LEFT JOIN thickness_variant as t on t.productId=p.id
                            ${conditionQuery} group by p.id;`
            //(sql);
            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        } catch (e) {
            return e;
        }
    }

    async findProductByMenu(routeName: string, take: number, page: number, categoryId: String,
        brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        try {
            if (routeName === "new_arrival") {
                //(take);
                //(page);
                let whereQuery: any = ``;
                if (categoryId) {
                    whereQuery = `p.firstCategoryId = '${categoryId}' `;
                }
                if (brand) {
                    var result = '(';
                    if (brand.length > 0) {
                        brand = brand.split(",")
                    }
                    //(brand);
                    brand.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and p.brandName IN ${result} `;
                    } else {
                        whereQuery += ` p.brandName IN ${result} `;
                    }

                }
                if (maxPrice && minPrice) {
                    if (whereQuery.length > 3) {
                        whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    } else {
                        whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    }

                }
                // if (color) {
                //     var result = '(';
                //     if (color.length > 0) {
                //         color = color.split(",")
                //     }
                //     color.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += ` and c.color IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` c.color IN ${result} `;
                //     }

                // }
                // if (size) {
                //     var result = '(';
                //     if (size.length > 0) {
                //         size = size.split(",")
                //     }
                //     size.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += `and s.size IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` s.size IN ${result} `;
                //     }

                // }
                if (thickness) {
                    var result = '(';
                    if (thickness.length > 0) {
                        thickness = thickness.split(",")
                    }
                    thickness.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and t.thickness IN ${result} `;
                    } else {
                        whereQuery += ` t.thickness IN ${result} `;
                    }

                }

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.isDeleted=0 `;
                } else {
                    whereQuery += ` p.isDeleted=0 `;
                }

                if (whereQuery.substring(0, 3) === 'and') {
                    whereQuery = whereQuery.slice(4, whereQuery.length);
                }

                let conditionQuery = ``;

                if (!whereQuery) {
                    conditionQuery = ``;
                } else {
                    conditionQuery = ` WHERE ${whereQuery} `;
                }

                let offset = 0;


                if (page > 1) {
                    offset = ((page - 1) * take);
                }


                let sql = `SELECT p.* FROM new_arrival as p 
                            LEFT JOIN thickness_variant as t on t.productId=p.id
                             ${conditionQuery} GROUP BY p.id limit ${offset}, ${take};`
                let sqlTotal = `SELECT p.* FROM new_arrival as p 
                                  LEFT JOIN thickness_variant as t on t.productId=p.id
                                  ${conditionQuery} group by p.id;`
                //(sql);
                //(sqlTotal);
                const entityManager = getManager();
                const data = await entityManager.query(sql);
                const totalData = await entityManager.query(sqlTotal);
                const count = totalData.length;


                //('Repo = ', routeName);
                const pageCount = Math.ceil(count / take);

                const responseData = {
                    page: page,
                    limit: take,
                    pageCount: pageCount,
                    totalDataFound: count,
                    data
                };

                return responseData;
            }
            if (routeName === "best_selling") {
                //(take);
                //(page);
                let whereQuery: any = ``;
                if (categoryId) {
                    whereQuery = `p.firstCategoryId = '${categoryId}' `;
                }
                if (brand) {
                    var result = '(';
                    if (brand.length > 0) {
                        brand = brand.split(",")
                    }
                    //(brand);
                    brand.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and p.brandName IN ${result} `;
                    } else {
                        whereQuery += ` p.brandName IN ${result} `;
                    }

                }
                if (maxPrice && minPrice) {
                    if (whereQuery.length > 3) {
                        whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    } else {
                        whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                    }

                }
                // if (color) {
                //     var result = '(';
                //     if (color.length > 0) {
                //         color = color.split(",")
                //     }
                //     color.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += ` and c.color IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` c.color IN ${result} `;
                //     }

                // }
                // if (size) {
                //     var result = '(';
                //     if (size.length > 0) {
                //         size = size.split(",")
                //     }
                //     size.forEach(item => {
                //         result += `'${item}', `;
                //     })
                //     result = result.substring(0, result.length - 2);
                //     result += ')';
                //     if (whereQuery.length > 3) {
                //         whereQuery += `and s.size IN ${result} `;
                //     }
                //     else {
                //         whereQuery += ` s.size IN ${result} `;
                //     }

                // }
                if (thickness) {
                    var result = '(';
                    if (thickness.length > 0) {
                        thickness = thickness.split(",")
                    }
                    thickness.forEach(item => {
                        result += `'${item}', `;
                    })
                    result = result.substring(0, result.length - 2);
                    result += ')';
                    if (whereQuery.length > 3) {
                        whereQuery += ` and t.thickness IN ${result} `;
                    } else {
                        whereQuery += ` t.thickness IN ${result} `;
                    }

                }

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.isDeleted=0 `;
                } else {
                    whereQuery += ` p.isDeleted=0 `;
                }

                if (whereQuery.substring(0, 3) === 'and') {
                    whereQuery = whereQuery.slice(4, whereQuery.length);
                }

                let conditionQuery = ``;

                if (!whereQuery) {
                    conditionQuery = ``;
                } else {
                    conditionQuery = ` WHERE ${whereQuery} `;
                }

                let offset = 0;


                if (page > 1) {
                    offset = ((page - 1) * take);
                }


                let sql = `SELECT p.* FROM best_selling as p 
                            LEFT JOIN thickness_variant as t on t.productId=p.id
                            ${conditionQuery} group by p.id  limit ${offset}, ${take};`
                let sqlTotal = `SELECT p.* FROM best_selling as p 
                                  LEFT JOIN thickness_variant as t on t.productId=p.id
                                  ${conditionQuery} group by p.id;`
                //(sql);
                //(sqlTotal);
                const entityManager = getManager();
                const data = await entityManager.query(sql);
                const totalData = await entityManager.query(sqlTotal);
                const count = totalData.length;


                //('Repo = ', routeName);
                const pageCount = Math.ceil(count / take);

                const responseData = {
                    page: page,
                    limit: take,
                    pageCount: pageCount,
                    totalDataFound: count,
                    data
                };

                return responseData;
            }
        } catch (e) {
            return e;
        }
    }

    async findProductByChildCategory(categoryId: string, take: number, page: number, brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        try {
            //(take);
            //(page);
            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.thirdCategoryId = '${categoryId}' `;
            }
            if (brand) {
                var result = '(';
                if (brand.length > 0) {
                    brand = brand.split(",")
                }
                //(brand);
                brand.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.brandName IN ${result} `;
                } else {
                    whereQuery += ` p.brandName IN ${result} `;
                }

            }
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                } else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }

            }
            // if (color) {
            //     var result = '(';
            //     if (color.length > 0) {
            //         color = color.split(",")
            //     }
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += ` and c.color IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` c.color IN ${result} `;
            //     }

            // }
            // if (size) {
            //     var result = '(';
            //     if (size.length > 0) {
            //         size = size.split(",")
            //     }
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += `and s.size IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` s.size IN ${result} `;
            //     }

            // }
            if (thickness) {
                var result = '(';
                if (thickness.length > 0) {
                    thickness = thickness.split(",")
                }
                thickness.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.thickness IN ${result} `;
                } else {
                    whereQuery += ` t.thickness IN ${result} `;
                }

            }

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 `;
            } else {
                whereQuery += ` p.isDeleted=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }


            let sql = `SELECT p.* FROM products as p 
                        LEFT JOIN color_variant as c on c.productId=p.id
                        LEFT JOIN size_variant as s on s.productId=p.id
                        LEFT JOIN thickness_variant as t on t.productId=p.id
                        ${conditionQuery} group by p.id  limit ${offset}, ${take}; `
            let sqlTotal = `SELECT p.* FROM products as p 
                                LEFT JOIN color_variant as c on c.productId=p.id
                                LEFT JOIN size_variant as s on s.productId=p.id
                                LEFT JOIN thickness_variant as t on t.productId=p.id
                                ${conditionQuery} group by p.id;`
            //(sql);
            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        } catch (e) {
            return e;
        }
    }

    async findProductByParentCategory(categoryId: string, take: number, page: number, brand: any,
        maxPrice: number,
        minPrice: number,
        thickness: any): Promise<any | Error> {
        try {

            //(take);
            //(page);
            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.firstCategoryId = '${categoryId}' `;
            }
            if (brand) {
                var result = '(';
                if (brand.length > 0) {
                    brand = brand.split(",")
                }
                //(brand);
                brand.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.brandName IN ${result} `;
                } else {
                    whereQuery += ` p.brandName IN ${result} `;
                }

            }
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                } else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }

            }
            // if (color) {
            //     var result = '(';
            //     if (color.length > 0) {
            //         color = color.split(",")
            //     }
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += ` and c.color IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` c.color IN ${result} `;
            //     }

            // }
            // if (size) {
            //     var result = '(';
            //     if (size.length > 0) {
            //         size = size.split(",")
            //     }
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     if (whereQuery.length > 3) {
            //         whereQuery += `and s.size IN ${result} `;
            //     }
            //     else {
            //         whereQuery += ` s.size IN ${result} `;
            //     }

            // }
            if (thickness) {
                var result = '(';
                if (thickness.length > 0) {
                    thickness = thickness.split(",")
                }
                thickness.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and t.thickness IN ${result} `;
                } else {
                    whereQuery += ` t.thickness IN ${result} `;
                }

            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 `;
            } else {
                whereQuery += ` p.isDeleted=0 `;
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }


            let sql = `SELECT p.* FROM products as p 
                        LEFT JOIN thickness_variant as t on t.productId=p.id
                        ${conditionQuery}  GROUP BY p.id limit ${offset}, ${take};`
            let sqlTotal = `SELECT p.* FROM products as p 
                                LEFT JOIN thickness_variant as t on t.productId=p.id
                                ${conditionQuery} group by p.id;`
            //(sql);
            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;
        } catch (e) {
            return e;
        }
    }

    async getProductsByQuery(productIds: any): Promise<Product[] | Error> {
        try {
            const products = await getConnection()
                .createQueryBuilder()
                .from(Product, "product")
                .where("product.id IN (:...ids)", { ids: productIds })
                .execute();

            return products;
        } catch (e) {
            return e;
        }
    }


    async findOneRawEntity(
        id: string
    ): Promise<any | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return false;
        }
    }

    async findOneBySlugEntity(
        slug: string
    ): Promise<any | Error> {
        try {
            return await this.findOne({ where: { urlSlug: slug } });
        } catch (e) {
            return false;
        }
    }

    async findOneEntity(id: string): Promise<any | Error> {
        let query: any = id;
        query.isDeleted = false;
        try {
            return await this.find({
                where: query,
            });
        } catch (e) {
            return e;
        }
    }

    async findOneProduct(
        id: string
    ): Promise<Product[] | Error> {
        let query: any = { id: id };
        query.isDeleted = false;
        try {
            return await this.find({
                where: query,
            },
            );
        } catch (e) {
            return e;
        }
    }

    async findOneProductBySlug(
        slug: string
    ): Promise<Product | Error> {

        let query: any = { urlSlug: slug };
        query.isDeleted = false;
        query.stock = MoreThan(0);
        try {
            return await this.findOne({
                where: query,
            },
            );
        } catch (e) {
            return e;
        }
    }

    async productsFilterByParams(
        limit: number,
        page: number,
        categoryId: String,
        brandName: any,
        maxPrice: number,
        minPrice: number,
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            if (categoryId) {
                whereQuery = `p.firstCategoryId = '${categoryId}'`;
            }
            if (brandName) {
                var result = '(';
                brandName.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                whereQuery += `and p.brandName IN ${result}`;
            }
            if (maxPrice && minPrice) {
                whereQuery += `and (p.price >= ${minPrice} and p.price <= ${maxPrice})`;
            }
            // if (color) {
            //     var result = '(';
            //     color.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     whereQuery += `and c.color IN ${result}`;
            // }
            // if (size) {
            //     var result = '(';
            //     size.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     whereQuery += `and s.size IN ${result}`;
            // }
            // if (thickness) {
            //     var result = '(';
            //     thickness.forEach(item => {
            //         result += `'${item}', `;
            //     })
            //     result = result.substring(0, result.length - 2);
            //     result += ')';
            //     whereQuery += `and t.thickness IN ${result}`;
            // }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            if (!whereQuery) {
                whereQuery = `1`;
            }

            let sql = `SELECT p.id, p.name, p.price, p.brandName, p.featuredImage
                        FROM products p
                        WHERE ${whereQuery}
                        ORDER BY p.name;`

            const entityManager = getManager();
            const query = await entityManager.query(sql);

            return query;
        } catch (e) {
            return e;
        }
    }

    async filterByProductsName(
        take: any,
        page: any,
        keyword: any,
        parentCategoryId: any,
        maxPrice: any,
        minPrice: any,
        color: any,
        size: any,
        sale: any,
        sortBy: any,
        readyToShip: any,
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            // sortBy operation
            if (!sortBy) {
                orderByQuery = ` ORDER BY p.createdAt DESC `;
            }

            if (sortBy === 'new') {
                orderByQuery = ` ORDER BY p.createdAt DESC `;
            }

            if (sortBy === 'high') {
                orderByQuery = ` ORDER BY p.price DESC`;
            }

            if (sortBy === 'low') {
                orderByQuery = ` ORDER BY p.price ASC`;
            }

            if (sortBy === 'topRating') {
                orderByQuery = ` ORDER BY p.averageRating DESC`;
            }

            if (sortBy === 'lowRating') {
                orderByQuery = ` ORDER BY p.averageRating ASC`;
            }


            // keyword operation searh by name and child category name and parent category name
            if (keyword) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%'`;
                } else {
                    whereQuery += ` p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%'`;
                }
            }

            // filter using parent category id
            if (parentCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.parentCategoryId = '${parentCategoryId}' `;
                } else {
                    whereQuery += ` p.parentCategoryId = '${parentCategoryId}' `;
                }

            }

            // filter Ready to ship
            if (readyToShip) {
                if (whereQuery.length > 3) {
                    whereQuery += ` AND (p.isReadyToShip =1 AND p.stock > 0) `;
                } else {
                    whereQuery += ` (p.isReadyToShip =1 AND p.stock > 0) `;
                }
            }

            // filter using max price and min price
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                } else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
            }

            // filter using color
            if (color) {
                var result = '(';
                if (color.length > 0) {
                    color = color.split(",")
                }
                color.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and pcv.colorCode IN ${result} `;
                } else {
                    whereQuery += ` pcv.colorCode IN ${result} `;
                }

            }

            // filter using size
            if (size) {
                //
                var result = '(';
                if (size.length > 0) {
                    size = size.split(",")
                }
                size.forEach(item => {
                    //
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and psv.sizeName IN ${result} `;
                } else {
                    whereQuery += ` psv.sizeName IN ${result} `;
                }

            }

            // filter using sale
            if (sale) {
                var result = '(';
                if (sale.length > 0) {
                    sale = sale.split(",")
                }
                sale.forEach(item => {
                    result += `"${item}", `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and psalevar.saleName IN ${result} `;
                } else {
                    whereQuery += ` psalevar.saleName IN ${result} `;
                }

            }


            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 and p.isDeactive=0 `;
            } else {
                whereQuery += ` p.isDeleted=0 and p.isDeactive=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = `WHERE p.stock > 0`;
            } else {
                conditionQuery = ` WHERE ${whereQuery} AND p.stock > 0`;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }


            let sql = `SELECT p.*,
            (SELECT GROUP_CONCAT(pcv.colorCode) AS colorCode
             FROM product_color_variants AS pcv
             WHERE pcv.productId = p.id) AS colorCode,
            (SELECT GROUP_CONCAT(pcv.colorName) AS colorName
             FROM product_color_variants AS pcv
             WHERE pcv.productId = p.id) AS colorName,
            (SELECT GROUP_CONCAT(psv.sizeName) AS sizeName
             FROM product_size_variants AS psv
             WHERE psv.productId = p.id) AS sizeName,
            (SELECT GROUP_CONCAT(psalevar.saleName) AS saleName
             FROM product_sale_variants AS psalevar
             WHERE psalevar.productId = p.id) AS saleName
          FROM products AS p
          LEFT JOIN product_color_variants AS pcv ON p.id = pcv.productId
          LEFT JOIN product_size_variants AS psv ON p.id = psv.productId
          LEFT JOIN product_sale_variants AS psalevar ON p.id = psalevar.productId
            ${conditionQuery} GROUP BY p.id ${orderByQuery} limit ${offset}, ${take};`

            let sqlTotal = `SELECT COUNT(*) FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_sale_variants as psalevar ON p.id = psalevar.productId
            ${conditionQuery} GROUP BY p.id;`

            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / take);

            const responseData = {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;

        } catch (e) {
            return e;
        }
    }


    async discountedProductWithColor(
        take: any,
        page: any): Promise<any | Error> {
        try {
            //
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            // sortBy operation

            // filter using parent category id
            whereQuery += ` p.discountPrice > 0 AND p.isDeleted=0`;
            let conditionQuery = ``;
            conditionQuery = ` WHERE ${whereQuery} `;
            let offset = 0;
            if (page > 1) {
                offset = ((page - 1) * take);
            }

            orderByQuery = `ORDER BY p.createdAt DESC `;


            let sql = `SELECT p.*, (SELECT GROUP_CONCAT(CONCAT(colorCode)) AS colorCode FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode  FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            ${conditionQuery} GROUP BY p.id ${orderByQuery}  limit ${offset}, ${take};`

            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);

            return data;

        } catch (e) {
            return [];
        }
    }

    async discountedProductWithColorCount(): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            whereQuery += ` p.discountPrice > 0 AND p.isDeleted=0`;
            let conditionQuery = ``;
            conditionQuery = ` WHERE ${whereQuery} `;

            let sqlTotal = `SELECT p.* FROM products as p
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId 
            ${conditionQuery} GROUP BY p.id ;`
            const entityManager = getManager();
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;

            return count;

        } catch (e) {
            return e;
        }
    }

    async getProductByCategory(categoryId: string): Promise<any | Error> {
        try {

            return await getRepository(Product)
                .createQueryBuilder()
                .select("Product.id")
                .where("firstCategoryId = :categoryId OR secondCategoryId = :categoryId OR thirdCategoryId = :categoryId", {
                    categoryId: categoryId
                })
                .getMany();
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(Product)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countProduct(): Promise<any | Error> {
        try {
            return await this.count({
                where: { isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async todayProducts(): Promise<any | Error> {
        const start = new Date();
        start.setUTCHours(0, 0, 0, 0);
        const end = new Date();
        end.setUTCHours(23, 59, 59, 999);

        const data = await this.find({
            where: {
                isDeleted: false,
                createdAt: Between(
                    new Date(start).toISOString(),
                    new Date(end).toISOString()
                )
            },
            relations: [
                'productParentCategory',
                'productsubCategory',
                'productChildCategory',
                'productImages',
                'productColorVariants',
                'productSizeVariants',
                'productTagVariants',
                'productMaterialVariants',
                'productHighlightVariants'
            ]
        });

        return data;
    }

    async bestSelling(): Promise<any | Error> {
        const sql = `SELECT p.* FROM best_selling as p WHERE p.isDeleted=false ORDER BY p.soldQuantity DESC LIMIT 8;`
        const entityManager = getManager();
        const data = await entityManager.query(sql);
        return data;
    }

    // async filterRelatedProductEntities(subCategoryId: string): Promise<any | Error> {
    //     try {
    //         return await this.find({
    //             take: 9,
    //             order: {createdAt: -1},
    //             where: [{subCategoryId: subCategoryId, isDeleted: false}],
    //             relations: ['photos', 'productsubCategory', 'productParentCategory']
    //         });
    //     } catch (e) {
    //         return e;
    //     }
    // }

    async getRelatedProducts(productId: string, categoryId: string): Promise<any | Error> {
        try {
            const sql = `SELECT p.*, 
            (SELECT GROUP_CONCAT(CONCAT(pcv.colorCode)) AS colorCode
            FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode,
            (SELECT GROUP_CONCAT(CONCAT(psv.sizeName)) AS sizeName 
            FROM product_size_variants as psv where psv.productId=p.id) as sizeName,
            (SELECT GROUP_CONCAT(CONCAT(psalevar.saleName)) AS saleName 
            FROM product_sale_variants as psalevar where psalevar.productId=p.id) as saleName
            FROM products p WHERE p.parentCategoryId = '${categoryId}'
            AND p.id != '${productId}'
            AND p.isDeleted = 0 AND p.stock > 0
            ORDER BY p.createdAt DESC;`;
            const relatedProducts = await this.manager.query(sql);

            return {
                relatedProducts
            }

        } catch (e) {

            return e;
        }

    }

    async findLatestProducts(rowCount: number): Promise<any | Error> {
        try {
            const sql = `SELECT p.*, 
            (SELECT GROUP_CONCAT(CONCAT(pcv.colorCode)) AS colorCode
            FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode,
            (SELECT GROUP_CONCAT(CONCAT(psv.sizeName)) AS sizeName 
            FROM product_size_variants as psv where psv.productId=p.id) as sizeName,
            (SELECT GROUP_CONCAT(CONCAT(psalevar.saleName)) AS saleName 
            FROM product_sale_variants as psalevar where psalevar.productId=p.id) as saleName
            FROM products p WHERE
            p.isDeleted = 0 AND p.stock > 0
            ORDER BY p.createdAt DESC LIMIT ${rowCount}`;
            const latestProducts = await this.manager.query(sql);

            return latestProducts;


        } catch (e) {

            return e;
        }

    }

    async findPopularProducts(categories: ProductCategory[], rowCount: number): Promise<any | Error> {
        const categoryIds = categories.map(category => `'${category.id}'`).join(', ');;
        try {
            const sql = `SELECT p.*, 
            (SELECT GROUP_CONCAT(CONCAT(pcv.colorCode)) AS colorCode
            FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode,
            (SELECT GROUP_CONCAT(CONCAT(psv.sizeName)) AS sizeName 
            FROM product_size_variants as psv where psv.productId=p.id) as sizeName,
            (SELECT GROUP_CONCAT(CONCAT(psalevar.saleName)) AS saleName 
            FROM product_sale_variants as psalevar where psalevar.productId=p.id) as saleName
            FROM products p
            LEFT JOIN product_category category 
            ON p.parentCategoryId = category.id AND category.id IN(${categoryIds})
            WHERE p.isDeleted=0 AND p.stock > 0
            ORDER BY p.averageRating
            DESC LIMIT ${rowCount};`;

            const popularProducts = await this.manager.query(sql);

            return popularProducts;

        } catch (e) {

            return e;
        }

    }

    async findClearenceSaleProducts(saleId: string, rowCount: number): Promise<any | Error> {
        try {
            const sql = `SELECT p.*, 
            (SELECT GROUP_CONCAT(CONCAT(pcv.colorCode)) AS colorCode
            FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode,
            (SELECT GROUP_CONCAT(CONCAT(psv.sizeName)) AS sizeName 
            FROM product_size_variants as psv where psv.productId=p.id) as sizeName,
            (SELECT GROUP_CONCAT(CONCAT(psalevar.saleName)) AS saleName 
            FROM product_sale_variants as psalevar where psalevar.productId=p.id) as saleName
            FROM products p
            LEFT JOIN product_sale_variants psalevar 
            ON p.id = psalevar.productId
            WHERE p.isDeleted=0 AND p.stock > 0 AND psalevar.saleId = '${saleId}'
            ORDER BY p.averageRating
            DESC LIMIT ${rowCount};`;

            const clearenceSaleProducts = await this.manager.query(sql);

            return clearenceSaleProducts;

        } catch (e) {

            return e;
        }

    }

    async exportAllProductRepo(): Promise<Product[] | Error> {
        try {
            return await this.find(
                {
                    order: { createdAt: "DESC" },
                    where: { isDeleted: false },
                    relations: ['photos', 'productsubCategory', 'productParentCategory']
                },
            );
        } catch (e) {
            return e;
        }
    }


    async productReorderRepository(
        limit: number,
        page: number,
        categoryId: String
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;


            whereQuery = ` p.stock <= p.quantityReorder `;

            whereQuery += ` AND p.isDeleted=0 `;

            if (categoryId) {
                whereQuery += ` AND p.subCategoryId = '${categoryId}'`;
            }


            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * limit);
            }


            let sql = `SELECT p.* FROM products as p 
                        ${conditionQuery} GROUP BY p.id;`
            let sqlTotal = `SELECT p.* FROM products as p 
                                ${conditionQuery} group by p.id;`
            //(sql);
            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / limit);

            const responseData = {
                page: page,
                limit: limit,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

            return responseData;


        } catch (e) {
            return e;
        }
    }

    async topProducts(): Promise<any | Error> {
        try {
            const productData = await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    totalOrder: -1
                },
                take: 3
            });
            let labelArr = [];
            let dataArr = [];
            let dataArray = [];
            for (let i = 0; i < productData.length; i++) {
                const data = productData[i];
                labelArr.push(data.name);
                dataArr.push(data.totalOrder);
                // dataArray.push(arr);
                // arr = [];

            }
            return {
                labels: labelArr,
                data: dataArr
            };
        } catch (e) {
            return e;
        }
    }

    async topTwoProducts(): Promise<any | Error> {
        try {
            const productData = await this.find({
                where: {
                    isDeleted: false
                },
                order: {
                    totalOrder: -1
                },
                take: 2
            });
            // let labelArr = [];
            // let dataArr = [];
            // let dataArray = [];
            // for (let i = 0; i < productData.length; i++) {
            //     const data = productData[i];
            //     // labelArr.push(data.name);
            //     // dataArr.push(data.totalOrder);
            //     dataArray.push(data);
            //     // arr = [];

            // }
            // // return {
            // //     labels: labelArr,
            // //     data: dataArr
            // // };
            return productData;
        } catch (e) {
            return e;
        }
    }

    async productOnSale(
        take: any,
        page: any,
        keyword: any,
        routeName: any,
        parentCategoryId: any,
        subCategoryId: any,
        childCategoryId: any,
        maxPrice: any,
        minPrice: any,
        color: any,
        size: any,
        tag: any,
        material: any,
        highlight: any,
        sortBy: any,
        readyToShip: any,
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            if (sortBy === 'new') {
                orderByQuery = `ORDER BY p.createdAt DESC `;
            }

            if (sortBy === 'high') {
                orderByQuery = `ORDER BY p.price DESC`;
            }

            if (sortBy === 'low') {
                orderByQuery = `ORDER BY p.price ASC`;
            }

            // routeName operation
            if (routeName === 'merch') {
                whereQuery = `p.isMerch = 1 `;
            }

            if (routeName === 'giftGuide') {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.isGiftGuide = 1 `;
                } else {
                    whereQuery = `p.isGiftGuide = 1 `;
                }
            }

            // keyword operation search by name and child category name and parent category name
            if (keyword) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%' OR 
                    p.childCategoryName LIKE '%${keyword}%' `;
                } else {
                    whereQuery += ` p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%' OR 
                    p.childCategoryName LIKE '%${keyword}%' `;
                }
            }

            // filter using parent category id
            if (parentCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.parentCategoryId = '${parentCategoryId}' `;
                } else {
                    whereQuery += ` p.parentCategoryId = '${parentCategoryId}' `;
                }

            }

            // filter using sub category id
            if (subCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.subCategoryId = '${subCategoryId}' `;
                } else {
                    whereQuery += ` p.subCategoryId = '${subCategoryId}' `;
                }

            }

            // filter using child category id
            if (childCategoryId) {

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.childCategoryId = '${childCategoryId}' `;
                } else {
                    whereQuery += ` p.childCategoryId = '${childCategoryId}' `;
                }

            }

            // filter Ready to ship
            if (readyToShip) {
                if (whereQuery.length > 3) {
                    whereQuery += ` AND (p.isReadyToShip =1 AND p.stock > 0) `;
                } else {
                    whereQuery += ` (p.isReadyToShip =1 AND p.stock > 0) `;
                }
            }

            // filter using max price and min price
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                } else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
            }

            // filter using color
            if (color) {
                let result = '(';
                if (color.length > 0) {
                    color = color.split(",")
                }
                color.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and pcv.colorName IN ${result} `;
                } else {
                    whereQuery += ` pcv.colorName IN ${result} `;
                }

            }

            // filter using size
            if (size) {
                //
                let result = '(';
                if (size.length > 0) {
                    size = size.split(",")
                }
                size.forEach(item => {
                    //
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and psv.sizeName IN ${result} `;
                } else {
                    whereQuery += ` psv.sizeName IN ${result} `;
                }

            }

            // filter using tag
            if (tag) {
                let result = '(';
                if (tag.length > 0) {
                    tag = tag.split(",")
                }
                tag.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and ptv.tagName IN ${result} `;
                } else {
                    whereQuery += ` ptv.tagName IN ${result} `;
                }

            }

            // filter using material
            if (material) {
                let result = '(';
                if (material.length > 0) {
                    material = material.split(",")
                }
                material.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and pmv.materialName IN ${result} `;
                } else {
                    whereQuery += ` pmv.materialName IN ${result} `;
                }

            }

            // filter using highlight
            if (highlight) {
                let result = '(';
                if (highlight.length > 0) {
                    highlight = highlight.split(",")
                }
                highlight.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and phv.highlightName IN ${result} `;
                } else {
                    whereQuery += ` phv.highlightName IN ${result} `;
                }

            }

            if (whereQuery.length > 3) {
                whereQuery += ` and pos.isSale=1 and (pos.expireDate >= CURDATE() or pos.expireDate IS NULL) `;
            } else {
                whereQuery += ` pos.isSale=1 and (pos.expireDate >= CURDATE() or pos.expireDate IS NULL) `;
            }


            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 and p.isDeactive=0 `;
            } else {
                whereQuery += ` p.isDeleted=0 and p.isDeactive=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }

            let sql = `SELECT p.*, (SELECT GROUP_CONCAT(CONCAT(colorCode)) AS colorCode FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode  FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            LEFT JOIN product_on_sale as pos ON p.id = pos.productId
            ${conditionQuery} GROUP BY p.id  ${orderByQuery} limit ${offset}, ${take};`

            let sqlTotal = `SELECT p.* FROM products as p
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            LEFT JOIN product_on_sale as pos ON p.id = pos.productId 
            ${conditionQuery}  GROUP BY p.id ;`

            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / take);

            return {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

        } catch (e) {
            return e;
        }
    }


    async shopByOccasion(
        take: any,
        page: any,
        keyword: any,
        routeName: any,
        parentCategoryId: any,
        subCategoryId: any,
        childCategoryId: any,
        maxPrice: any,
        minPrice: any,
        color: any,
        size: any,
        tag: any,
        material: any,
        highlight: any,
        sortBy: any,
        readyToShip: any,
    ): Promise<any | Error> {
        try {
            let whereQuery: any = ``;
            let orderByQuery: any = ``;

            if (sortBy === 'new') {
                orderByQuery = `ORDER BY p.createdAt DESC `;
            }

            if (sortBy === 'high') {
                orderByQuery = `ORDER BY p.price DESC`;
            }

            if (sortBy === 'low') {
                orderByQuery = `ORDER BY p.price ASC`;
            }

            // routeName operation
            if (routeName === 'merch') {
                whereQuery = `p.isMerch = 1 `;
            }

            if (routeName === 'giftGuide') {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.isGiftGuide = 1 `;
                } else {
                    whereQuery = `p.isGiftGuide = 1 `;
                }
            }

            // keyword operation search by name and child category name and parent category name
            if (keyword) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%' OR 
                    p.childCategoryName LIKE '%${keyword}%' `;
                } else {
                    whereQuery += ` p.name like '%${keyword}%' OR 
                    p.parentCategoryName LIKE '%${keyword}%' OR 
                    p.childCategoryName LIKE '%${keyword}%' `;
                }
            }

            // filter using parent category id
            if (parentCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.parentCategoryId = '${parentCategoryId}' `;
                } else {
                    whereQuery += ` p.parentCategoryId = '${parentCategoryId}' `;
                }

            }

            // filter using sub category id
            if (subCategoryId) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and p.subCategoryId = '${subCategoryId}' `;
                } else {
                    whereQuery += ` p.subCategoryId = '${subCategoryId}' `;
                }

            }

            // filter using child category id
            if (childCategoryId) {

                if (whereQuery.length > 3) {
                    whereQuery += ` and p.childCategoryId = '${childCategoryId}' `;
                } else {
                    whereQuery += ` p.childCategoryId = '${childCategoryId}' `;
                }

            }

            // filter Ready to ship
            if (readyToShip) {
                if (whereQuery.length > 3) {
                    whereQuery += ` AND (p.isReadyToShip =1 AND p.stock > 0) `;
                } else {
                    whereQuery += ` (p.isReadyToShip =1 AND p.stock > 0) `;
                }
            }

            // filter using max price and min price
            if (maxPrice && minPrice) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                } else {
                    whereQuery += ` (p.price >= ${minPrice} and p.price <= ${maxPrice}) `;
                }
            }

            // filter using color
            if (color) {
                let result = '(';
                if (color.length > 0) {
                    color = color.split(",")
                }
                color.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += ` and pcv.colorName IN ${result} `;
                } else {
                    whereQuery += ` pcv.colorName IN ${result} `;
                }

            }

            // filter using size
            if (size) {
                //
                let result = '(';
                if (size.length > 0) {
                    size = size.split(",")
                }
                size.forEach(item => {
                    //
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and psv.sizeName IN ${result} `;
                } else {
                    whereQuery += ` psv.sizeName IN ${result} `;
                }

            }

            // filter using tag
            if (tag) {
                let result = '(';
                if (tag.length > 0) {
                    tag = tag.split(",")
                }
                tag.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and ptv.tagName IN ${result} `;
                } else {
                    whereQuery += ` ptv.tagName IN ${result} `;
                }

            }

            // filter using material
            if (material) {
                let result = '(';
                if (material.length > 0) {
                    material = material.split(",")
                }
                material.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and pmv.materialName IN ${result} `;
                } else {
                    whereQuery += ` pmv.materialName IN ${result} `;
                }

            }

            // filter using highlight
            if (highlight) {
                let result = '(';
                if (highlight.length > 0) {
                    highlight = highlight.split(",")
                }
                highlight.forEach(item => {
                    result += `'${item}', `;
                })
                result = result.substring(0, result.length - 2);
                result += ')';
                if (whereQuery.length > 3) {
                    whereQuery += `and phv.highlightName IN ${result} `;
                } else {
                    whereQuery += ` phv.highlightName IN ${result} `;
                }

            }

            if (whereQuery.length > 3) {
                whereQuery += ` and sbo.isOccasion=1 and (sbo.expireDate >= CURDATE() or sbo.expireDate IS NULL) `;
            } else {
                whereQuery += ` sbo.isOccasion=1 and (sbo.expireDate >= CURDATE() or sbo.expireDate IS NULL) `;
            }


            if (whereQuery.length > 3) {
                whereQuery += ` and p.isDeleted=0 and p.isDeactive=0 `;
            } else {
                whereQuery += ` p.isDeleted=0 and p.isDeactive=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;


            if (page > 1) {
                offset = ((page - 1) * take);
            }

            let sql = `SELECT p.*, (SELECT GROUP_CONCAT(CONCAT(colorCode)) AS colorCode FROM product_color_variants as pcv where pcv.productId=p.id) as colorCode  FROM products as p 
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            LEFT JOIN shop_by_occasion as sbo ON p.id = sbo.productId
            ${conditionQuery} GROUP BY p.id  ${orderByQuery} limit ${offset}, ${take};`

            let sqlTotal = `SELECT p.* FROM products as p
            LEFT JOIN product_color_variants as pcv ON p.id = pcv.productId
            LEFT JOIN product_size_variants as psv ON p.id = psv.productId
            LEFT JOIN product_tag_variants as ptv ON p.id = ptv.productId
            LEFT JOIN product_material_variants as pmv ON p.id = pmv.productId
            LEFT JOIN product_highlight_variants as phv ON p.id = phv.productId
            LEFT JOIN shop_by_occasion as sbo ON p.id = sbo.productId 
            ${conditionQuery}  GROUP BY p.id ;`

            //(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData.length;
            const pageCount = Math.ceil(count / take);

            return {
                page: page,
                limit: take,
                pageCount: pageCount,
                totalDataFound: count,
                data
            };

        } catch (e) {
            return e;
        }
    }

    async paginate(page: number, limit: number): Promise<any | Error> {
        try {
            let offset = 0;

            if (page > 1) {
                offset = (page - 1) * limit;
            }

            const productSQL = `SELECT product.*, category.name as productCategoryName, category.urlSlug as productCategoryUrlSlug,
            category.image as productCategoryImage
            FROM products product
            INNER JOIN product_category category
            ON product.parentCategoryId = category.id
            WHERE product.isDeleted=0 ORDER BY product.createdAt 
            DESC LIMIT ${limit} OFFSET ${offset};`;

            const productCountSQL = `SELECT count(*) as count FROM products WHERE isDeleted=0;`;

            const productListCount = await this.manager.query(productCountSQL);
            const productList = await this.manager.query(productSQL);

            const totalPages = Math.ceil(productListCount?.[0]?.count / limit);
            const nextPage = page < totalPages ? page + 1 : null;
            const previousPage = page > 1 ? page - 1 : null;

            return {
                products: productList,
                totalCount: productListCount[0].count,
                totalPages,
                nextPage,
                previousPage
            }

        } catch (e) {
            return e;
        }


    }

    async getAllForAdmin(): Promise<any | Error> {
        try {
            const productQuery = `SELECT product.*, category.name as productCategoryName, category.urlSlug as productCategoryUrlSlug,
            category.image as productCategoryImage
            FROM products product
            INNER JOIN product_category category
            ON product.parentCategoryId = category.id
            WHERE product.isDeleted=0
            ORDER BY product.createdAt DESC;`;

            let products = await this.manager.query(productQuery);

            const productIds = products.map((x: { id: string }) => `"${x.id}"`).join(",");

            const sizeVarientQuery = `select * from product_size_variants where productId in (${productIds})`;
            const sizeVarients = await this.manager.query(sizeVarientQuery);
            const sizeVarientsMap = new Map<string, any[]>();
            for (const item of sizeVarients) {
                if (sizeVarientsMap.has(item.productId)) {
                    const varient = sizeVarientsMap.get(item.productId);
                    sizeVarientsMap.set(item.productId, [...varient, item]);
                } else {
                    sizeVarientsMap.set(item.productId, [item]);
                }
            }

            const colorVarientQuery = `select * from product_color_variants where productId in (${productIds})`;
            const colorVarients = await this.manager.query(colorVarientQuery);
            const colorVarientsMap = new Map<string, any[]>();
            for (const item of colorVarients) {
                if (colorVarientsMap.has(item.productId)) {
                    const varient = colorVarientsMap.get(item.productId);
                    colorVarientsMap.set(item.productId, [...varient, item]);
                } else {
                    colorVarientsMap.set(item.productId, [item]);
                }
            }

            products = products.map((x: any) => {
                let sizeVariants = [];
                if (sizeVarientsMap.get(x.id)) {
                    sizeVariants = sizeVarientsMap.get(x.id);
                }

                let colorVariants = [];
                if (colorVarientsMap.get(x.id)) {
                    colorVariants = colorVarientsMap.get(x.id);
                }
                return {
                    ...x,
                    sizeVariants,
                    colorVariants
                }
            });

            return products;

        } catch (e) {
            return e;
        }
    }

}