import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {ResponseUtils} from 'src/utils/response.utils';
import {Constants} from '../../utils/constants';
import {FileUploadsService} from '../file-uploads/file-uploads.service';
import {SendEmailService} from '../send-email/send-email.service';
import {UploadedMulterFileI} from '../SpacesModule/SpacesService';
import {DoSpacesService} from '../SpacesModule/SpacesService/doSpacesService';
import {CreateUserDto} from './dto/create-user.dto';
import {UpdateUserDto} from './dto/update-user.dto';
import {User} from './entities/user.entity';
import {UsersRepository} from './user.repository';
import {MailSenderService} from '../mail-sender/mail-sender.service';
import {EmailNotifierRepository} from '../email-notifier/email-notifier-repository';

@Injectable()
export class UsersService {
    constructor(
        @InjectRepository(UsersRepository)
        private readonly repository: UsersRepository,
        private readonly sendEmailService: SendEmailService,
        private readonly fileService: FileUploadsService,
        private readonly doSpaceService: DoSpacesService,
        private readonly servMail: MailSenderService,
        private readonly emailNotifierRepository: EmailNotifierRepository,
    ) {
    }

    async create(dto: CreateUserDto): Promise<User | Error> {
        const email = dto.email;

        const isEmailExists = await this.repository.findByEmailAddress(email)
        if (isEmailExists) {
            throw new HttpException(
                "Email Already Exists",
                HttpStatus.BAD_REQUEST
            );
        }
        const data = await this.repository.createEntity(dto);

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Admin created successfully!", "data", data);
        // if (dto.userType === "admin") {
        //     //user getting email
        //     let mailDto = {
        //         to: [email],
        //         from: process.env.EMAIL_SENDER_MAIL,
        //         subject: 'Hyman Fashion Admin Account Creation Notification',
        //         name: dto.firstName + " " + dto.lastName,
        //         email: email
        //     };

        //     const userMail = await this.servMail.accocuntCreateAdminNotify(mailDto as CreateAdminMailSenderDto);
        //    
        //     const admins: any = await this.emailNotifierRepository.findOnlyEmail();

        //     if (admins && admins.length > 0) {
        //         admins.forEach(async (element: any) => {
        //             let mailAdminDto = {
        //                 to: [element.email],
        //                 from: process.env.EMAIL_SENDER_MAIL,
        //                 subject: 'New Admin Notification.',
        //                 name: dto.firstName + " " + dto.lastName,
        //                 email: email
        //             };
        //             const adminMail = await this.servMail.adminNotify(
        //                 mailAdminDto as AdminNotifyDto,
        //             );
        //            
        //         });
        //     }

        //     return ResponseUtils.successResponseHandler(201, "Admin created successfully!", "data", data);
        // }
        // else {
        //     //user getting email
        //     let mailDto = {
        //         to: [email],
        //         from: process.env.EMAIL_SENDER_MAIL,
        //         subject: 'Hyman Fashion User Account Creation Notification',
        //         username: dto.firstName + " " + dto.lastName,
        //         password: dto.password,
        //         email: email
        //     };

        //     const userMail = await this.servMail.accocuntCreateByAdminCustomerNotify(mailDto as CreateMailSenderDto);
        //    
        //     const admins: any = await this.emailNotifierRepository.findOnlyEmail();

        //     if (admins && admins.length > 0) {
        //         admins.forEach(async (element: any) => {
        //             let mailAdminDto = {
        //                 to: [element.email],
        //                 from: process.env.EMAIL_SENDER_MAIL,
        //                 subject: 'New Customer Notification.',
        //                 name: dto.firstName + " " + dto.lastName,
        //                 email: email
        //             };
        //             const adminMail = await this.servMail.adminNotify(
        //                 mailAdminDto as AdminNotifyDto,
        //             );
        //            
        //         });
        //     }

        //     return ResponseUtils.successResponseHandler(201, "User created successfully!", "data", data);
        // }
    }

    async findAll(userType?: string): Promise<User[] | Error> {
        const data = await this.repository.findByQuery(userType);

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        let userList = [];
        for (let i = 0; i < Object.keys(data).length; i++) {
            if (data[i].email !== "guest@hymanfashion.com") {
                userList.push(data[i]);
            }
        }
        return ResponseUtils.successResponseHandler(200, "Users data fetched!", "data", userList);
    }

    async findOne(id: string): Promise<User | Error> {
        const data = await this.repository.findOneEntity(id);

        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "User data fetched!", "data", data);
    }

    async update(id: string, dto: UpdateUserDto): Promise<User | Error> {
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "User updated sucessfully!", "data", data);
    }

    async updateUserProfile(request: any, dto: UpdateUserDto, files: {
        image: UploadedMulterFileI
    }): Promise<User | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "profileImages/");
            dto.image = image;
        } else {
            dto.image = dto.image;
        }
        const data = await this.repository.updateEntity(request.user.id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(200, "User profile updated successfully!", "data", data);
    }


    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.buildDeletedData(
            data,
            200,
            "User deleted successfully!"
        );
    }
}
